//
//  UserInfoOperator.m
//
//  Created by Z  on 16/5/11
//  Copyright (c) 2016 __MyCompanyName__. All rights reserved.
//

#import "UserInfoOperator.h"


NSString *const kUserInfoOperatorId = @"id";
NSString *const kUserInfoOperatorFullName = @"fullName";


@interface UserInfoOperator ()

- (id)objectOrNilForKey:(id)aKey fromDictionary:(NSDictionary *)dict;

@end

@implementation UserInfoOperator

@synthesize operatorIdentifier = _operatorIdentifier;
@synthesize fullName = _fullName;


+ (instancetype)modelObjectWithDictionary:(NSDictionary *)dict
{
    return [[self alloc] initWithDictionary:dict];
}

- (instancetype)initWithDictionary:(NSDictionary *)dict
{
    self = [super init];
    
    // This check serves to make sure that a non-NSDictionary object
    // passed into the model class doesn't break the parsing.
    if(self && [dict isKindOfClass:[NSDictionary class]]) {
            self.operatorIdentifier = [self objectOrNilForKey:kUserInfoOperatorId fromDictionary:dict];
            self.fullName = [self objectOrNilForKey:kUserInfoOperatorFullName fromDictionary:dict];

    }
    
    return self;
    
}

- (NSDictionary *)dictionaryRepresentation
{
    NSMutableDictionary *mutableDict = [NSMutableDictionary dictionary];
    [mutableDict setValue:self.operatorIdentifier forKey:kUserInfoOperatorId];
    [mutableDict setValue:self.fullName forKey:kUserInfoOperatorFullName];

    return [NSDictionary dictionaryWithDictionary:mutableDict];
}

- (NSString *)description 
{
    return [NSString stringWithFormat:@"%@", [self dictionaryRepresentation]];
}

#pragma mark - Helper Method
- (id)objectOrNilForKey:(id)aKey fromDictionary:(NSDictionary *)dict
{
    id object = [dict objectForKey:aKey];
    return [object isEqual:[NSNull null]] ? nil : object;
}


#pragma mark - NSCoding Methods

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super init];

    self.operatorIdentifier = [aDecoder decodeObjectForKey:kUserInfoOperatorId];
    self.fullName = [aDecoder decodeObjectForKey:kUserInfoOperatorFullName];
    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{

    [aCoder encodeObject:_operatorIdentifier forKey:kUserInfoOperatorId];
    [aCoder encodeObject:_fullName forKey:kUserInfoOperatorFullName];
}

- (id)copyWithZone:(NSZone *)zone
{
    UserInfoOperator *copy = [[UserInfoOperator alloc] init];
    
    if (copy) {

        copy.operatorIdentifier = [self.operatorIdentifier copyWithZone:zone];
        copy.fullName = [self.fullName copyWithZone:zone];
    }
    
    return copy;
}


@end
